//****************************************************************
//* LPM4.5 Wake up test
//****************************************************************
// Description:
//   This is an example code to demonstrate the usage of  LPM4.5 and
//   some technics for debugging during this state
//
//   Falling edge on P1.6 enters LPM4.5
//   Falling edge on P1.7 exits LPM4.5
//
//                MSP430F5xxx
//             -----------------
//         /|\|              XIN|-
//          | |                 |
//          --|RST          XOUT|-
//            |                 |
//            |             P1.0|-->LED
// Sw P1.6 -->|P1.6             |
// Sw P1.7 -->|P1.7             |
//
//  S.Schauer
//  Texas Instruments Inc.
//  Aug 2009
//  Built with CCS Version: 4.0.0 and IAR Embedded Workbench Version: 4.11B
//******************************************************************************

#include  "msp430.h"

unsigned int lpm4_5_flag = 0;               // Flag to mark LPM4.5 request

void initPorts(void)
{
  PAOUT = 0x00C0;
  PADIR = 0x0001;                           // Enable Driver for LED
  PAREN = 0xFFFE;                           // Enable Pull down/pull-up Rs
  PAIES = BIT6+BIT7;                        // Select Falling Edge
  PAIFG = 0x0000;                           // Clear all pending interrupt Flags
  PAIE  = BIT6;                             // Enable interrupt on PA.6 = P1.6
#ifdef __MSP430_HAS_PORTB_R__
  PBOUT = 0x0000;
  PBDIR = 0xFFFF;                           // Enalbe OUTPUT driver
#endif
#ifdef __MSP430_HAS_PORTC_R__
  PCOUT = 0x0000;
  PCREN = 0xFFFF;                           // Enalbe Pull down
#endif
#ifdef __MSP430_HAS_PORTD_R__
  PDOUT = 0x0000;
  PDREN = 0xFFFF;                           // Enalbe Pull down
#endif
#ifdef __MSP430_HAS_PORTE_R__
  PEOUT = 0x0000;
  PEREN = 0xFFFF;                           // Enalbe Pull down
#endif
#ifdef __MSP430_HAS_PORTF_R__
  PFOUT = 0x0000;
  PFREN = 0xFFFF;                           // Enalbe Pull down
#endif
  PJOUT = 0xFFFF;
  PJREN = 0xFFFF;                           // Enalbe Pull up

}

void LPM4_5_WakeUP(void)
{
  volatile unsigned int i = 0x1000;
  // Note:
  // The following line should only be included for debugging the LPM4.5 wake up
  while ((PAIN & 0x0080) == 0);             // SW trap for LPM4.5 -- execution  will stop here as long as input is Low
                                            // This could be used to synch up the debugger and continue to debug from here after LPM4.5

  while(i--);                               // wait some time for debouncing of switches
  initPorts();                              // reinitilize the ports before the LOCKIO bit will be cleared
  PBOUT = 0x0008;                           // wakeup request from LPM4.5 detected
  PMMCTL0_H = PMMPW_H;                      // open PMM
  PM5CTL0 &= ~LOCKIO;                       // Clear LOCKIO and enable ports
  PMMCTL0_H = 0x00;                         // close PMM

  PAIFG = 0x0000;                           // Clear all pending interrupt Flags - do this (again) after the lock has been removed !!!!

}

void enter_LPM4_5(void)
{
  __enable_interrupt();                     // ensure that interrupts are enabled
  PMMCTL0_H = PMMPW_H;                      // open PMM
  PMMCTL0_L |= PMMREGOFF;                   // set Flag to enter LPM4.5 with LPM4 request
  LPM4;                                     // now enter LPM4.5
}

void main(void)
{
  unsigned int lpm4_5_wakeup_flag = 0;      // flag to indicate wake up from LPM4.5
  unsigned int i = 0;
  lpm4_5_flag = 0;

  WDTCTL = WDTPW + WDTHOLD;                 // Stop Watchdog

#ifdef __MSP430_HAS_USB__
  USBKEYPID = USBKEY;                       // open the USB
  USBPWRCTL = 0;                            // disable the LDOs
#endif

  while (i == 0)
  {
    switch (__even_in_range(SYSRSTIV,SYSRSTIV_PMMKEY))
    {
    case SYSRSTIV_NONE:
      i = 1;                                // stop loop - all resets are executed
      break;
    case SYSRSTIV_BOR:
      PBOUT = 0x0001;
      break;
    case SYSRSTIV_RSTNMI:
      break;
    case SYSRSTIV_DOBOR:
      break;
    case SYSRSTIV_LPM5WU:
      LPM4_5_WakeUP();
      PBOUT = 0x0010;
      lpm4_5_wakeup_flag = 1;
      break;
    case SYSRSTIV_SECYV:
      break;
    case SYSRSTIV_SVSL:
      break;
    case SYSRSTIV_SVSH:
      break;
    case SYSRSTIV_SVML_OVP:
      break;
    case SYSRSTIV_SVMH_OVP:
      break;
    case SYSRSTIV_DOPOR:
      break;
    case SYSRSTIV_WDTTO:
      break;
    case SYSRSTIV_WDTKEY:
      break;
    case SYSRSTIV_KEYV:
      break;
    case SYSRSTIV_FLLUL:
      break;
    case SYSRSTIV_PERF:
      break;
    case SYSRSTIV_PMMKEY:
      break;
    }
  }

  // start normal startup routine and code execution
  if (lpm4_5_wakeup_flag == 0)                // check for wake up from LPM4.5
  {
    initPorts();
    __enable_interrupt();
    LPM4;
    _NOP();
  }

  __enable_interrupt();
  while (1)
  {
    PBOUT ^= 0x0002;
    if (lpm4_5_flag == 1)
    {
      enter_LPM4_5();                       // enter LPM4.5
    }
  }
}

#pragma vector=PORT1_VECTOR
__interrupt void Port1InterruptHandler(void)
{
    switch (__even_in_range(P1IV,P1IV_P1IFG7))
    {
    case P1IV_NONE:
      break;
    case P1IV_P1IFG0:
      break;
    case P1IV_P1IFG1:
      break;
    case P1IV_P1IFG2:
      break;
    case P1IV_P1IFG3:
      break;
    case P1IV_P1IFG4:
      break;
    case P1IV_P1IFG5:
      break;
    case P1IV_P1IFG6:                       // port 1.6 detected
        PBOUT = 0x0004;
        PAIE  &= ~BIT6;                     // Disable interrupt on PA.6 = P1.6
        PAIE  |= BIT7;                      // Enable interrupt on PA.7 = P1.7
        P1OUT |= BIT0;                      // switch on LED to show LPM4.5 state
        lpm4_5_flag = 1;                    // Set LPM4.5 request flag
        LPM4_EXIT;                          // Stay in active mode after ISR
      break;
    case P1IV_P1IFG7:
      break;
    }
}
